/*
 * Copyright (c) 1995-2014, Index Datassss
 * All rights reserved.
 * See the file LICENSE for details.
 */
package com.indexdata.torus2;

import com.indexdata.utils.MDCThreadPoolExecutor;
import java.util.Date;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;
import org.apache.log4j.Logger;

/**
 *
 * @author jakub
 */
public class StartupListener implements ServletContextListener {
  private final static Logger logger = Logger.getLogger("com.indexdata.torus2");
  public final static String STARTUP_DATE_ATTR = "startupDate";
  public final static String EXECUTOR_SERVICE_ATTR = "executorService";

  @Override
  public void contextInitialized(ServletContextEvent sce) {
    sce.getServletContext().setAttribute(STARTUP_DATE_ATTR, new Date());
    sce.getServletContext().setAttribute(EXECUTOR_SERVICE_ATTR, 
      MDCThreadPoolExecutor.newCachedThreadPool());
  }

  @Override
  public void contextDestroyed(ServletContextEvent sce) {
    sce.getServletContext().removeAttribute(STARTUP_DATE_ATTR);
    ExecutorService es = (ExecutorService) sce.getServletContext().getAttribute(EXECUTOR_SERVICE_ATTR);
    if (es != null && !es.isTerminated()) {
      logger.info("Found non-terminated executor service in context, shutting down...");
      es.shutdownNow();
      try {
        es.awaitTermination(10, TimeUnit.SECONDS);
      } catch (InterruptedException ie) {
        logger.warn("Executor service shutdown attempt terminated, giving up.");
      }
      if (!es.isTerminated()) {
        logger.warn("Executor service shutdown attempt timed out, giving up.");
      } else {
        logger.info("Executor service has shutdown.");
      }
    }
    
  }
  
}
