/*
 * Copyright (c) 1995-2012, Index Datassss
 * All rights reserved.
 * See the file LICENSE for details.
 */
package com.indexdata.torus2.query;

import org.z3950.zing.cql.CQLAndNode;
import org.z3950.zing.cql.CQLBooleanNode;
import org.z3950.zing.cql.CQLNode;
import org.z3950.zing.cql.CQLNotNode;
import org.z3950.zing.cql.CQLOrNode;
import org.z3950.zing.cql.CQLTermNode;

import com.indexdata.torus2.query.match.AndMatcher;
import com.indexdata.torus2.query.match.AndNotMatcher;
import com.indexdata.torus2.query.match.AnyFieldMatcher;
import com.indexdata.torus2.query.match.FixedFieldMatcher;
import com.indexdata.torus2.query.match.LayerMatcher;
import com.indexdata.torus2.query.match.OrMatcher;
import com.indexdata.torus2.query.match.TrueMatcher;
import com.indexdata.torus2.query.match.TypeAwareTermMatcher;

/**
 *
 * @author jakub
 */
public class MatcherBuilder {
  public static LayerMatcher build(CQLNode query) throws QueryException {
    if (query == null) return null;
    if (query instanceof CQLTermNode) {
      CQLTermNode tn = (CQLTermNode) (query);
      String index = tn.getIndex();
      if (index == null || index.isEmpty()
        //above sould never happen as the parser always sets empty index to serverChoice
        || index.equals("cql.serverChoice")
        || index.equals("cql.anyIndexes")
        || index.equals("cql.allIndexes")
        || index.equals("cql.keywords")) {
        index = "cql.anywhere";
        //certain empty queries return all, for sorting
        if (tn.getTerm() == null || tn.getTerm().isEmpty())
          return new TrueMatcher();
      } else if (index.equals("cql.allRecords")) {
        return new TrueMatcher();
      }
      if (index.equals("cql.anywhere")) {
        return new AnyFieldMatcher(new TypeAwareTermMatcher(tn, index));
      } else {
        return new FixedFieldMatcher(new TypeAwareTermMatcher(tn, index), index);
      }
    } else if (query instanceof CQLBooleanNode) {
      CQLBooleanNode bn = (CQLBooleanNode) (query);
      if (query instanceof CQLAndNode) {
        return new AndMatcher(build(bn.getLeftOperand()), build(bn.
          getRightOperand()));
      } else if (query instanceof CQLOrNode) {
        return new OrMatcher(build(bn.getLeftOperand()), build(bn.
          getRightOperand()));
      } else if (query instanceof CQLNotNode) {  // 'and-not'
        return new AndNotMatcher(build(bn.getLeftOperand()), build(bn.
          getRightOperand()));
      } else {
        throw new QueryException("CQL error: unsupported boolean '"
          + bn.toCQL() + "'");
      }
    } else {
      String q = query != null ? query.toCQL() : "null";
      throw new QueryException("CQL error: unsupported CQL '"
        + q + "'");
    }
  }
}
