/*
 * Copyright (c) 1995-2012, Index Datassss
 * All rights reserved.
 * See the file LICENSE for details.
 */
package com.indexdata.torus2.query.match.term;

import com.indexdata.torus2.query.QueryException;
import com.indexdata.torus2.query.match.TermMatcher;
import org.apache.log4j.Logger;

/**
 *
 * @author jakub
 */
public class IpEnclosesMatcher implements TermMatcher {
  private static Logger logger = Logger.getLogger("com.indexdata.torus2");
  private long ip;
  private String ipString;

  public IpEnclosesMatcher(String term) throws QueryException {
    //support IPv4-mapped IPv6 addresses, in queries only not records
    if (term.startsWith("::ffff:"))
      term = term.substring(7);
    ipString = term;
    String[] va = term.split("\\.");
    if (va.length != 4) {
      throw new QueryException("CQL Error: malformed IP address '" + term + "'");
    }
    ip = 0;
    try {
      for (int i = 0; i < 4; i++) {
        ip = 256 * ip + Integer.parseInt(va[i]);
      }
    } catch (NumberFormatException nfe) {
      throw new QueryException("CQL error: malformed IP address '"+term+"'");
    }
  }

  @Override
  public boolean match(String value) {
    String[] ranges = value.split(" +");
    if (logger.isDebugEnabled()) {
      logger.debug("Found " + ranges.length + " IP ranges in record");
    }
    //try to parse at least one range
    for (String range : ranges) {
      String[] values = range.split("\\-");
      String low, high;
      if (values.length == 1) {
        low = values[0];
        high = values[0];
      } else if (values.length == 2) {
        low = values[0];
        high = values[1];
      } else {
        logger.warn("Bad IP range in record '" + range
          + "' (len=" + values.length + "), ignored");
        continue;
      }
      String[] lo = low.split("\\.");
      String[] hi = high.split("\\.");
      if ((lo.length != 4) || (hi.length != 4)) {
        logger.warn("Malformed IPs in record: '" + low + "' - '" + high
          + "', ignored");
        continue;
      }
      long lov = 0;
      long hiv = 0;
      try {
        for (int i = 0; i < 4; i++) {
          lov = 256 * lov + Integer.parseInt(lo[i]);
          hiv = 256 * hiv + Integer.parseInt(hi[i]);
        }
      } catch (NumberFormatException nfe) {
        logger.warn("Malformed IPs in record: '" + low + "' - '" + high
          + "', ignored");
        continue;
      }
      if ((lov <= ip) && (ip <= hiv)) {
        if (logger.isDebugEnabled()) {
          logger.debug("IP range in record '" + lo + "'-'" + hi + "' "
            + "matched against term '" + ipString + "'");
        }
        return true;
      }
    }
    // none of the ranges matched, the whole thing is a mismatch
    if (logger.isDebugEnabled()) {
      logger.debug("IP range term '" + ipString + "' did not match '"
        + value + "'");
    }
    return false;
  }
}
