/*
 * Copyright (c) 1995-2012, Index Datassss
 * All rights reserved.
 * See the file LICENSE for details.
 */
package com.indexdata.torus2.query.match.term;

import com.indexdata.torus2.query.QueryException;
import com.indexdata.torus2.query.match.TermMatcher;
import org.apache.log4j.Logger;

/**
 *
 * @author jakub
 */
public class IpWithinMatcher implements TermMatcher {
  private String low;
  private String high;
  private long lov;
  private long hiv;
  private static Logger logger = Logger.getLogger("com.indexdata.torus2");

  public IpWithinMatcher(String term) throws QueryException {
    String[] terms = term.split("[^0-9.]+");
    if (terms.length != 2) {
      throw new QueryException(
        "CQL error: relation 'within' requires two IP addresses, not '"
        + term + "'");
    }
    low = terms[0];
    high = terms[1];
    String[] lo = low.split("\\.");
    String[] hi = high.split("\\.");
    if ((lo.length != 4) || (hi.length != 4)) {
      throw new QueryException("CQL error: malformed IP range: '" + low + "' - '" + high + "' ");
    }
    try {
      for (int i = 0; i < 4; i++) {
        lov = 256 * lov + Integer.parseInt(lo[i]);
        hiv = 256 * hiv + Integer.parseInt(hi[i]);
      }
    } catch (NumberFormatException nfe) {
      throw new QueryException("CQL error: malformed IP address in range: '" + low + "' - '" + high + "' ");
    }
  }

  @Override
  public boolean match(String value) {
    String[] va = value.split("\\.");
    if (va.length != 4) {
      logger.warn("Malformed IP in record: '" + value + "', ignored");
      return false;
    }
    long vav = 0;
    try {
      for (int i = 0; i < 4; i++) {
        vav = 256 * vav + Integer.parseInt(va[i]);
      }
    } catch (NumberFormatException nfe) {
      logger.warn("Malformed IP in record: '" + value + "', ignored");
      return false;
    }
    if (logger.isDebugEnabled()) {
      logger.debug("Matching IP: " + "lo='" + low + "' " + "hi='" + high + "' "
      + "value='" + value + "' ");
    }
    return ((lov <= vav) && (vav <= hiv));
  }
}
