/*
 * Copyright (c) 1995-2011, Index Data
 * All rights reserved.
 * See the file LICENSE for details.
 */

package com.indexdata.torus2.util;

import java.util.LinkedList;

/**
 *
 * @author jakub
 */
public class XMLWriter {
  private LinkedList<String> tags = new LinkedList<String>();
  private StringBuilder sb;
  private boolean inTag = false;

  public XMLWriter(StringBuilder sb) {
    this.sb = sb;
  }

  public XMLWriter tag(String name) {
    if (inTag)
      sb.append(">");
    tags.push(name);
    sb.append("<").append(name);
    inTag = true;
    return this;
  }

  public XMLWriter end() {
    if (tags.size() == 0)
      throw new IllegalStateException("'end' must be preceded by at least one 'tag'");
    String tag = tags.pop();
    if (inTag)
      sb.append("/>");
    else
      sb.append("</").append(tag).append(">");
    inTag = false;
    return this;
  }

  public XMLWriter attr(String name, String value) {
    if (!inTag)
      throw new IllegalStateException("'attr' must be directly preceded by 'tag'");
    sb.append(" ").append(name).append("=\"");
    escape(value);
    sb.append("\"");
    return this;
  }

  public XMLWriter text(String value) {
    if (tags.size() == 0)
      throw new IllegalStateException("'end' must be preceded by at least one 'tag'");
    if (inTag)
      sb.append(">");
    inTag = false;
    escape(value);
    return this;
  }

  private void escape(String value) {
    for(int i = 0; i<value.length(); i++) {
      char c = value.charAt(i);
      switch (c) {
        case '<':
          sb.append("&lt;");
          break;
        case '>':
          sb.append("&gt;");
          break;
        case '"':
          sb.append("&quot;");
          break;
        case '\'':
          sb.append("&apos;");
          break;
        case '&':
          sb.append("&amp;");
          break;
        default:
          sb.append(c);
      }
    }
  }

}
